﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Newtonsoft.Json;
using System.IO;

namespace MapEd
{
    public partial class Form1 : Form
    {
        private TileStrip tiles;
        private string filename;

        public Form1()
        {
            InitializeComponent();

            drawingMode.SelectedIndex = 0;
        }

        private void OnQuit(object sender, EventArgs e)
        {
            if (MessageBox.Show("Quit", "Quit", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.OK)
            {
                this.Close();
            }
        }

        private void EnableControls()
        {
            drawingMode.Enabled = true;
            tileNumber.Enabled = true;
            dataNumber.Enabled = true;
            tileMode.Enabled = true;
            dataMode.Enabled = true;
            tilePreview.Enabled = true;
        }

        private void OnNew(object sender, EventArgs e)
        {
            try
            {
                NewForm newForm = new NewForm();

                if (newForm.ShowDialog() == DialogResult.OK)
                {
                    if (tiles != null)
                    {
                        tiles.Dispose();
                        tiles = null;
                    }

                    tiles = new TileStrip(newForm.TileMap, newForm.TileSize);

                    tileNumber.Maximum = tiles.Count - 1;
                    tileNumber.Value = 0;
                    OnTileNumber(tileNumber, new EventArgs());

                    Tuple<int, int> size = newForm.MapSize;

                    map.SetMap(size.Item2, size.Item1, tiles, newForm.TileSize);

                    saveMenuItem.Enabled = false;
                    saveAsMenuItem.Enabled = true;

                    EnableControls();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnLoad(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog fsel = new OpenFileDialog();

                fsel.Filter = "MapEd Files (*.maped)|*.maped|All Files (*.*)|*.*";

                if (fsel.ShowDialog() == DialogResult.OK)
                {
                    filename = fsel.FileName;

                    using(StreamReader file = File.OpenText(filename))
                    {
                        string json = file.ReadToEnd();
                        FileContents contents = JsonConvert.DeserializeObject<FileContents>(json);

                        if (tiles != null)
                        {
                            tiles.Dispose();
                            tiles = null;
                        }

                        tiles = new TileStrip(contents.Strip, contents.TileSize);

                        tileNumber.Maximum = tiles.Count - 1;
                        tileNumber.Value = 0;
                        OnTileNumber(tileNumber, new EventArgs());

                        map.SetMap(contents.Width, contents.Height, tiles, contents.TileSize);
                        map.SetMap(contents.Map);
                        map.SetData(contents.Data);

                        file.Close();
                    }
                }

                saveMenuItem.Enabled = true;
                saveAsMenuItem.Enabled = true;
                EnableControls();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                filename = null;
            }
        }

        private void SaveMap()
        {
            FileContents contents = new FileContents();

            contents.Strip = tiles.SourceFile;
            contents.Width = map.MapWidth;
            contents.Height = map.MapHeight;
            contents.TileSize = map.TileSize;
            contents.Map = map.MapTiles;
            contents.Data = map.MapData;

            string json = JsonConvert.SerializeObject(contents);

            using (StreamWriter file = File.CreateText(filename))
            {
                file.Write(json);
                file.Close();
            }
        }

        private void OnSaveAs(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog fsel = new SaveFileDialog();
                fsel.Filter = "MapEd Files (*.maped)|*.maped|All Files (*.*)|*.*";

                if (fsel.ShowDialog() == DialogResult.OK)
                {
                    filename = fsel.FileName;
                    SaveMap();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnSave(object sender, EventArgs e)
        {
            try
            {
                if (!String.IsNullOrEmpty(filename))
                {
                    SaveMap();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnTileNumber(object sender, EventArgs e)
        {
            int no = (int)tileNumber.Value;

            tilePreview.BackgroundImage = tiles.GetImage(no);
            map.SetTileNumber(no);
        }

        private void OnDrawingMode(object sender, EventArgs e)
        {
            MapControl.DrawMode mode = (MapControl.DrawMode)drawingMode.SelectedIndex;
            map.MapDrawMode = mode;
        }

        private void OnDataNumber(object sender, EventArgs e)
        {
            map.SetDataNumber((int)dataNumber.Value);
        }

        private void OnDataOrTileMode(object sender, EventArgs e)
        {
            map.SetDataMode(sender == dataMode);
        }

        private void OnSelectTile(object sender, EventArgs e)
        {
            TileSelector selector = new TileSelector(tiles);

            if (selector.ShowDialog() == DialogResult.OK)
            {
                tileNumber.Value = selector.SelectedTile;
            }
        }
    }
}
